/******************************************************************************/
/* POS header file.                                                           */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/

#ifndef __POS_h
#define __POS_h

////////////////////////////////////////////////////////////////////////////////
// Set the operation system you compile under to one of these values:
//
// __p_beos     - BeOS
// __p_windows  - Microsoft Windows
// __p_linux    - Linux
////////////////////////////////////////////////////////////////////////////////
#define __p_beos			1
#define __p_windows			2
#define __p_linux			3

#define __p_os				__p_beos		// This is the one to change

// BeOS headers
#if __p_os == __p_beos

#include "BeOS.h"

#endif

// Windows headers
#if __p_os == __p_windows

#include "Windows/WindowsIncludes.h"

#endif


////////////////////////////////////////////////////////////////////////////////
// Set the endianess of your system you compile under to one of these values:
//
// __p_little   - Little endian
// __p_big      - Big endian
////////////////////////////////////////////////////////////////////////////////
#define __p_little			1
#define __p_big				2

#if __p_os == __p_beos
#ifdef __POWERPC__
#define __p_endian			__p_big
#else
#define __p_endian			__p_little
#endif
#endif

#if __p_os == __p_windows
#define __p_endian			__p_little
#endif

#if __p_os == __p_linux
fejl
// Hej John,
// Kører du med big endian eller little endian? Hvis du har begge muligheder
// ligesom ved BeOS, så skal du checke det på en eller anden måde og #define'e
// __p_endian til den rigtige værdi.
#endif


////////////////////////////////////////////////////////////////////////////////
// Datatype definition (BeOS datatypes)
//
// Use these datatypes everywhere in your add-on. This makes sure it can be
// compiled under any platform without any troubles.
////////////////////////////////////////////////////////////////////////////////
#if (__p_os == __p_windows) || (__p_os == __p_linux)

typedef signed char						int8;
typedef unsigned char					uint8;
typedef volatile signed char			vint8;
typedef volatile unsigned char			vuint8;

typedef short							int16;
typedef unsigned short					uint16;
typedef volatile short					vint16;
typedef volatile unsigned short			vuint16;

typedef long							int32;
typedef unsigned long					uint32;
typedef volatile long					vint32;
typedef volatile unsigned long			vuint32;

typedef __int64							int64;
typedef unsigned __int64				uint64;
typedef volatile __int64				vint64;
typedef volatile unsigned __int64		vuint64;

typedef volatile long					vlong;
typedef volatile int					vint;
typedef volatile short					vshort;
typedef volatile char					vchar;

typedef volatile unsigned long			vulong;
typedef volatile unsigned int			vuint;
typedef volatile unsigned short			vushort;
typedef volatile unsigned char			vuchar;

typedef unsigned char					uchar;
typedef unsigned short					unichar;

#endif


////////////////////////////////////////////////////////////////////////////////
// Max/min macros
////////////////////////////////////////////////////////////////////////////////
#if __p_os == __p_beos

#undef min
#undef max
#define min(x, y)						min_c(x, y)
#define max(x, y)						max_c(x, y)

#endif


////////////////////////////////////////////////////////////////////////////////
// Endian macros
////////////////////////////////////////////////////////////////////////////////
#if __p_os == __p_beos

// Well, under BeOS, we just use Be's own macros
#define P_HOST_TO_LENDIAN_INT16(arg)	B_HOST_TO_LENDIAN_INT16(arg)
#define P_HOST_TO_LENDIAN_INT32(arg)	B_HOST_TO_LENDIAN_INT32(arg)

#define P_HOST_TO_BENDIAN_INT16(arg)	B_HOST_TO_BENDIAN_INT16(arg)
#define P_HOST_TO_BENDIAN_INT32(arg)	B_HOST_TO_BENDIAN_INT32(arg)

#define P_LENDIAN_TO_HOST_INT16(arg)	B_LENDIAN_TO_HOST_INT16(arg)
#define P_LENDIAN_TO_HOST_INT32(arg)	B_LENDIAN_TO_HOST_INT32(arg)

#define P_BENDIAN_TO_HOST_INT16(arg)	B_BENDIAN_TO_HOST_INT16(arg)
#define P_BENDIAN_TO_HOST_INT32(arg)	B_BENDIAN_TO_HOST_INT32(arg)

#else

// Windows and Linux platforms
#if __p_endian == __p_little

#define P_HOST_TO_LENDIAN_INT16(arg)	arg
#define P_HOST_TO_LENDIAN_INT32(arg)	arg

#define P_HOST_TO_BENDIAN_INT16(arg)	((arg & 0xFF00) >> 8) | ((arg & 0xFF) << 8)
#define P_HOST_TO_BENDIAN_INT32(arg)	((arg & 0xFF000000) >> 24) | ((arg & 0xFF0000) >> 8) | ((arg & 0xFF00) <<8 ) | ((arg & 0xFF) << 24)

#define P_LENDIAN_TO_HOST_INT16(arg)	arg
#define P_LENDIAN_TO_HOST_INT32(arg)	arg

#define P_BENDIAN_TO_HOST_INT16(arg)	P_HOST_TO_BENDIAN_INT16(arg)
#define P_BENDIAN_TO_HOST_INT32(arg)	P_HOST_TO_BENDIAN_INT32(arg)

#endif

#endif


////////////////////////////////////////////////////////////////////////////////
// Directory slashes
////////////////////////////////////////////////////////////////////////////////
#if (__p_os == __p_beos) || (__p_os == __p_linux)
#define P_DIRSLASH		"/"
#define P_DIRSLASH_C	'/'
#endif

#if __p_os == __p_windows
#define P_DIRSLASH		"\\"
#define P_DIRSLASH_C	'\\'
#endif


////////////////////////////////////////////////////////////////////////////////
// Error codes base lines
////////////////////////////////////////////////////////////////////////////////
#define PGeneralErrorBase		(LONG_MIN)
#define PFileErrorBase			(PGeneralErrorBase + 0x1000)
#define PSecurityErrorBase		(PFileErrorBase + 0x1000)
#define PKeyErrorBase			(PSecurityErrorBase + 0x1000)


////////////////////////////////////////////////////////////////////////////////
// General errors
////////////////////////////////////////////////////////////////////////////////
enum PGeneralError
{
	P_OK						= 0,					// No errors
	P_ERR_ANY					= -1,					// Any error

	P_GEN_ERR_BAD_ARGUMENT		= PGeneralErrorBase,	// Invalid argument passed to function
	P_GEN_ERR_NO_MEMORY,								// Out of memory
	P_GEN_ERR_NO_RESOURCES,								// Out of resources
	P_GEN_ERR_NO_INIT,									// Can't perform the operation. Missing proper initialization.
	P_GEN_ERR_CANT_PERFORM,								// Can't perform the operation.
	P_GEN_ERR_BAD_IDENTIFIER,							// Invalid identifier
	P_GEN_ERR_BAD_INDEX,								// Invalid index value
	P_GEN_ERR_UNSUPPORTED_OS,							// Unsupported operative system
	P_GEN_ERR_INSUFFICIENT_BUFFER,						// The data area passed to a function is too small
};


////////////////////////////////////////////////////////////////////////////////
// File errors
////////////////////////////////////////////////////////////////////////////////
enum PFileError
{
	P_FILE_ERR_FILE				= PFileErrorBase,		// Illegal file operation
	P_FILE_ERR_NAME_TOO_LONG,							// File or directory name too long
	P_FILE_ERR_ENTRY_NOT_FOUND,							// Entry could not be found
	P_FILE_ERR_ENTRY_EXISTS,							// Entry does already exists
	P_FILE_ERR_DISK_FULL,								// There is not enough space on the disk
	P_FILE_ERR_NOT_A_DIRECTORY,							// The directory is invalid
	P_FILE_ERR_IS_A_DIRECTORY,							// File operation on a directory
	P_FILE_ERR_SEEK,									// An attempt was made to move the file pointer before the beginning of the file
	P_FILE_ERR_EOF,										// EOF reached
	P_FILE_ERR_LINK,									// Link error in the file system
	P_FILE_ERR_LOCKED,									// File or directory is locked
	P_FILE_ERR_FILE_CORRUPT,							// File is corrupted
	P_FILE_ERR_INVALID_ENTRY_NAME,						// Entry name is invalid
	P_FILE_ERR_BAD_ACCESS_MODE,							// Invalid access mode
};


////////////////////////////////////////////////////////////////////////////////
// Security errors
////////////////////////////////////////////////////////////////////////////////
enum PSecurityError
{
	P_SEC_ERR_ACCESS_DENIED     = PSecurityErrorBase,	// Access denied
};


////////////////////////////////////////////////////////////////////////////////
// Key errors
////////////////////////////////////////////////////////////////////////////////
enum PKeyError
{
	P_KEY_ERR_ENTRY_EXIST		= PKeyErrorBase,		// An entry does already exist and cannot be overwritten
	P_KEY_ERR_ENTRY_NOT_FOUND,							// An entry could not be found
};

#endif
