/******************************************************************************/
/* PRegistry header file.                                                     */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/


#ifndef __PRegistry_h
#define __PRegistry_h

// PolyKit headers
#include "POS.h"
#include "PString.h"
#include "PFile.h"
#include "PSynchronize.h"
#include "ImportExport.h"


/******************************************************************************/
/* APRegistry class                                                           */
/******************************************************************************/
#if __p_os == __p_beos && __POWERPC__
#pragma export on
#endif

class _IMPEXP_PKLIB PRegistry
{
public:
	enum PRegKind {pKey, pData, pNumber, pString};

	PRegistry(void);
	virtual ~PRegistry(void);

	void *OpenKey(PString path);
	void CloseKey(void *handle);
	bool Exists(void *handle, PString name);

	void CreateKey(void *handle, PString name);
	void CreateData(void *handle, PString name, const void *dataAdr, uint32 size);
	void CreateNumber(void *handle, PString name, int32 num);
	void CreateString(void *handle, PString name, PString str);

	void RemoveKey(void *handle, PString name);
	void RemoveKeyEntries(void *handle, PString name);
	void RemoveValue(void *handle, PString name);

	void *GetData(void *handle, PString name, uint32 *size);
	int32 GetNumber(void *handle, PString name);
	PString GetString(void *handle, PString name);

	void *InitValueEnum(void *handle);
	void *GetNextValue(void *handle, PString &name, void **buffer, uint32 *size, PRegKind *type);
	void *InitKeyEnum(void *handle);
	void *GetNextKey(void *handle, PString &name);

	void CopyKey(PString source, PString destination, PRegistry *destReg = NULL);

	void LoadReg(PString company, PString program, PString fileName);
	void SaveReg(PString company, PString program, PString fileName);
	void CloneReg(PRegistry *source);

protected:
	typedef struct PRegistryData
	{
		struct PRegistryData *	next;
		uint32					openCnt;
		PRegKind				type;
		PString					name;
		void *					data;
		uint32					dataSize;
	} PRegistryData;

	typedef struct PDiskData
	{
		uint32					next;
		uint32					type;		// PRegKind type
		uint32					data;
		uint32					dataSize;
		uint32					nameLen;
	} PDiskData;

	PRegistryData *FindLast(PRegistryData *start);
	PRegistryData *FindEntry(PRegistryData *start, PString Name, PRegistryData **prevEntry = NULL);
	void CreateValue(PRegistryData *workKey, PString name, PRegKind type, const void *dataAdr, uint32 size);
	void DeleteKey(PRegistryData *curKey);
	void *GetValue(PRegistryData *workKey, PString name, uint32 *size);

	void LoadKey(PFile *file, int32 position, PRegistryData *curKey);
	void SaveKey(PFile *file, PRegistryData *curKey);
	void CopyKey(PString key, PRegistry *source);

	PRegistryData *root;
	PSWMRLocker *lock;
	bool changed;

	uint8 *dataBlock;
};

#if __p_os == __p_beos && __POWERPC__
#pragma export off
#endif

#endif
