/******************************************************************************/
/* PThread header file.                                                       */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/


#ifndef __PThread_h
#define __PThread_h

// PolyKit headers
#include "POS.h"
#include "PString.h"
#include "PSynchronize.h"
#include "ImportExport.h"


/******************************************************************************/
/* PThread class                                                              */
/******************************************************************************/
typedef int32 (*PThreadFunc)(void *);

#if __p_os == __p_beos && __POWERPC__
#pragma export on
#endif

class _IMPEXP_PKLIB PThread
{
public:
	enum PPriority {pIdle, pLow, pBelowNormal, pNormal, pAboveNormal, pHigh, pRealTime};

	PThread(void);
	virtual ~PThread(void);

	void StartThread(void);
	void WaitOnThread(void) const;
	int32 GetExitCode(void) const;

	void Resume(void);
	void Suspend(void);
	void Kill(void);

	bool IsAlive(void) const;

	void SetHookFunc(PThreadFunc func, void *userData = NULL);
	void SetName(PString name);
	void SetPriority(PPriority pri);
	PPriority GetPriority(void) const;

protected:
#if __p_os == __p_beos

	static int32 ThreadFunc(void *object);
	int32 ConvertPriority(void) const;

	thread_id tid;
	PEvent *suspendSignal;
	int32 suspendCounter;

#elif __p_os == __p_windows

	static DWORD WINAPI ThreadFunc(void *object);
	int ConvertPriority(void) const;

	HANDLE thread;		// Thread handle
	DWORD tid;			// Thread id
	PEvent threadStart;	// Thread start event

#endif

	bool threadRunning;
	PPriority threadPri;
	PString threadName;
	PThreadFunc threadFunc;
	void *threadData;
	int32 exitCode;
};

#if __p_os == __p_beos && __POWERPC__
#pragma export off
#endif

#endif
